__author__ = 'Joe Linn'

#import pylastica.filter.range
from .range import Range

class NumericRange(Range):
    """
    @see: http://www.elasticsearch.org/guide/reference/query-dsl/numeric-range-filter/
    """
    pass
